//------------------------------------------------------------------- DistrQEarnings.do
// This code produces Figure 3 (Article),
// that is, it draws the empirical distribution of quarterly earnings in the JF sample.
// 
// This code is embedded into a program to make the output (written to STATA Results' window) more readable.
//
// This code has four parts:
// 	PART 1: sets parameters/ preferences 
// 	PART 2: works on the data in entry to prepare it for PART 4
// 	PART 3: works on the data in entry to prepare it for PART 4
//	PART 4: graph the CDF of binned and re-centered earnings in the JF sample
//	PART 5: send code for execution 
//
// This code take as inputs:
// 	1. pre-generated MDRC data
// 	2. pre-generated data sets of welfare policy rules 
// 
// This code outputs:
// 	1. .pdf files with the CDFs of binned and re-centered earnings in the JF sample.
// 
// Last Revision: 09/04/2015
//-------------------------------------------------------------------------------------------------------------------------------
args jf_dtain rules_dtain




	
	
//----------------------------------------------------------------------
// Internal program
//----------------------------------------------------------------------
capture program drop DistrQEarnings
capture program define DistrQEarnings
	args jf_dtain rules_dtain
	
	
	
	
	
	//------------------------------------------------ PART 1  ------------------------------------------------------
	// Pick what you want to do by setting a few local variables:
	//
	// trimearnq         = $ amount above which you do not want to consider earnings when plotting histograms 
	//                     b/c outliers mess up the picture
	// nbinsaroundnotch  = number of bins around the notch that you want to plot in the histogram 
	//                     e.g. if nbinsaroundnotch=10 & width=100 then the histogram will consider 10 $100-bins below and 
	//                     10 $100-bins above bin 0 which is the bin centered around the notch or 
	//                     the convex-kink of choice width of the bins used when plotting twoway graph histogram
	//
	// NOTE: these are the values used in the paper, no other options have been vetted.
	//---------------------------------------------------------------------------------------------------------------
	local trimearnq = 10000 
	local nbinsaroundnotch = 10     
	local fq = 1
	local lq = 7
	local maxs = 12
	local maxy = 1999 // enough to go to Q7 post RA
	local whatwedo 	"Figure3"
	
	
	
	//------------------------------------------------ PART 2  ------------------------------------------------------
	// Policy Parameters.
	//---------------------------------------------------------------------------------------------------------------	
	qui use "`rules_dtain'", clear
	keep  F3 year size
	rename F3 F3_
	reshape wide F3, i(year) j(size)
	sort year 
	qui save "$ddJF/JF_ADFCrules_temp.dta", replace

	
	
	
	//------------------------------------------------ PART 3  ------------------------------------------------------
	// Use JF sample for post RA period 
	//---------------------------------------------------------------------------------------------------------------	
	qui use "`jf_dtain'" if e==1&quarter>=`fq', clear


	//------------------------------------------------ PART 4  ------------------------------------------------------
	// Infer AU size for JF-sample.
	// Create binned and centered earning variable to draw histograms in Part 3.
	//---------------------------------------------------------------------------------------------------------------

	//--------------------------------------------
	// AU size based on matching exactly the grant amount (aka MAP), treatment sample only.
	// Clearly, the MAP amount is available only in months when the case is on assistance.
	// The code below thus "spreads" over months when off welfare.
	// It also "spreads" over months when case is on welfare but grant not exactly equal to MAP.
	//
	// NOTES:
	//	a. 	We only consider AU sizes up to 8
	//
	//	b. 	e=0 identifies the control group for which we cannot use the grant amount to infer the AU size
	//      since the grant amount could be reduced due to earnings.
	//
	//	c. 	the unreduced grant amount (aka MAP) varies by whether or not the case received Housing Subsidies (HS)
	//     	AU's size		MAP if HS=0		MAP if HS=1
	//			1			333 [350]			306 [300]
	//			2			443 [450]			407 [400]
	//			3			543 [550]			500 [500]
	//			4			639 [650]			588 [600]
	//			5			731 [750]			672 [700]
	//			6			827 [850]			761 [750]
	//			7			933 [950]			858 [850]
	//			8			1031 [1050]			948 [950]
	// 		Because grant amounts are rounded to nearest $50 in MDRC's Public Use Files (see amounts reported in brakets), 
	// 		we face a situation in which e.g. an observed grant amount of $950 could correspond to a
	// 		AU size = 7 w/o HS or to a AU size = 9 with HS. Below we resolve this indeterminancy by assuming that
	// 		e.g. all of the $950 grants are for a AU size=7 w/o HS.
	//
	//	d. 	We believe that some of the very high sizes inferred from grant amounts are mistakes due
	//  	to overpayment in a few months(recall that we are limiting attention to units that at RA had 
	//  	1 or 2 kids so it is highly improbable that they ballooned to a AU size of 5,6,7, or 8
	//  	during the first 21 months); for this reason we do the following:
	//  	we replace the size of a given month if the size of the month before and/or after are smaller 
	//  	and we do this when the size of the month is one of the suspiciously high values of 5,6,7, or 8.
	//  	The details of this emerge from the code below.
	//--------------------------------------------

	// -----------------------------------------------------------------------------
	// inferring AU size using rounded grant amount (see explanation in a. through c. above)
	// -----------------------------------------------------------------------------
	qui gen hsizegrant = .
	qui replace hsizegrant = 1 if afdcamt==350|afdcamt==300
	qui replace hsizegrant = 2 if afdcamt==450|afdcamt==400
	qui replace hsizegrant = 3 if afdcamt==550|afdcamt==500
	qui replace hsizegrant = 4 if afdcamt==650|afdcamt==600
	qui replace hsizegrant = 5 if afdcamt==750|afdcamt==700 
	qui replace hsizegrant = 6 if afdcamt==850
	qui replace hsizegrant = 7 if afdcamt==950   
	qui replace hsizegrant = 8 if afdcamt==1050
	qui gen hsizegrant_orig = hsizegrant	
	// -----------------------------------------------------------------------------
	// Dealing with probable over-payments (see explanation in d. above)
	// Note: this code produces few changes, whether we apply it or not does not affect the results. 
	// -----------------------------------------------------------------------------
	qui gen hsizegrant_lag = .
	qui gen hsizegrant_lead = .
	sort pufid month
	qui bys pufid: replace hsizegrant_lag = hsizegrant[_n-1]
	qui bys pufid: replace hsizegrant_lead = hsizegrant[_n+1]
	qui replace hsizegrant = hsizegrant_lag if hsizegrant_lag<hsizegrant&hsizegrant_lead<hsizegrant& hsizegrant==5
	qui replace hsizegrant = hsizegrant_lag if hsizegrant_lag<hsizegrant&hsizegrant_lead<hsizegrant& hsizegrant==6
	qui replace hsizegrant = hsizegrant_lag if hsizegrant_lag<hsizegrant&hsizegrant_lead<hsizegrant& hsizegrant==7
	qui replace hsizegrant = hsizegrant_lag if hsizegrant_lag<hsizegrant&hsizegrant_lead<hsizegrant& hsizegrant==8
	qui replace hsizegrant = hsizegrant_lag if hsizegrant_lag<hsizegrant&hsizegrant_lead==.& hsizegrant==5
	qui replace hsizegrant = hsizegrant_lag if hsizegrant_lag<hsizegrant&hsizegrant_lead==.& hsizegrant==6
	qui replace hsizegrant = hsizegrant_lag if hsizegrant_lag<hsizegrant&hsizegrant_lead==.& hsizegrant==7
	qui replace hsizegrant = hsizegrant_lag if hsizegrant_lag<hsizegrant&hsizegrant_lead==.& hsizegrant==8
	qui replace hsizegrant = hsizegrant_lead if hsizegrant_lag==.&hsizegrant_lead<hsizegrant& hsizegrant==5
	qui replace hsizegrant = hsizegrant_lead if hsizegrant_lag==.&hsizegrant_lead<hsizegrant& hsizegrant==6
	qui replace hsizegrant = hsizegrant_lead if hsizegrant_lag==.&hsizegrant_lead<hsizegrant& hsizegrant==7
	qui replace hsizegrant = hsizegrant_lead if hsizegrant_lag==.&hsizegrant_lead<hsizegrant& hsizegrant==8
	qui replace hsizegrant = . if hsizegrant_lag==.&hsizegrant_lead==.& hsizegrant==5 
	qui replace hsizegrant = . if hsizegrant_lag==.&hsizegrant_lead==.& hsizegrant==6
	qui replace hsizegrant = . if hsizegrant_lag==.&hsizegrant_lead==.& hsizegrant==7
	qui replace hsizegrant = . if hsizegrant_lag==.&hsizegrant_lead==.& hsizegrant==8
	drop *_lag
	drop *_lead		
	// -----------------------------------------------------------------------------
	// After the replacements above, assume that when off AFDC the size is as when on until
	// the next time it is on again at which  point we start using the new size and so on 
	// -----------------------------------------------------------------------------
	sort pu month
	qui by pu: replace hsizegrant = hsizegrant[_n-1] if hsizegrant==.& hsizegrant[_n-1]~=.	
	// -----------------------------------------------------------------------------
	// Assume that when off AFDC at the start of the preRA period the size is as when on for the first time
	// -----------------------------------------------------------------------------
	foreach n of numlist 1(1)30 {
		sort id month
		qui by id: replace hsizegrant = hsizegrant[_n+1] if hsizegrant[_n]==.& hsizegrant[_n+1]~=.
	}	

	
	

	//---------------------------------------------------------------------------------------------- 
	// Bring in policy
	//----------------------------------------------------------------------------------------------	
	sort year
	qui merge m:1 year using  "$ddJF/JF_ADFCrules_temp.dta"
	qui drop if _m~=3 // drop years for which you do not have the rules
	drop _m	
	
	
	//---------------------------------------------------------------------------------------------- 
	// Retain only relevant post RA quarters
	//---------------------------------------------------------------------------------------------- 
	qui keep if quarter>=`fq'&quarter<=`lq'	
	

	//---------------------------------------------------------------------------------------------- 
	// Earnings binned in 100-wide bins. This is the built-in variable earnq provided by MDRC,
	// we only rest to missing all outliers.
	//----------------------------------------------------------------------------------------------	
	qui replace earnq = . if earnq>`trimearnq'
	//----------------------------------------------------------------------------------------------
	// For each observation determine in which bin the FPL on a quartely basis falls.
	// Recenter the binned earnings so that bin = 0 contains the FPL irrespective of the AU size and of calendar year.
	//----------------------------------------------------------------------------------------------
	qui gen Cbin = .
	forvalues y=1996(1)`maxy' {
		forvalues s=1(1)`maxs' {
			// identify the number of the bin to which the quartely FPL for an AU of this size in this calendar year
			qui replace Cbin =  0 if hsizegrant==`s'&year==`y'&3*F3_`s'==0 // for completeness, will never be active
			qui replace Cbin =  1 if hsizegrant==`s'&year==`y'&3*F3_`s'>0&3*F3_`s'<150 // for completeness
			qui replace Cbin =  round(3*F3_`s',100) if hsizegrant==`s'&year==`y'&Cbin==.
		}
	}
	assert Cbin~=. if hsizegrant<=`maxs'&year<=`maxy' 
	qui gen eCbin = earnq-Cbin 
	
	
	
	
	
	
	//------------------------------------------------ PART 4  ------------------------------------------------------
	// Draw distribution of binned and recentered earnings in the JF-sample:
	//	- using only quarters when case is off assistance in all 3 months of the quarter
	//	- using only quarters when case is on assistance in all 3 months of the quarter
	//	- using all quarters when case is either always off assistance or always on assistance
	//---------------------------------------------------------------------------------------------------------------	
	local mynotes  = "To match earning thresholds we use the AU size uniquely implied by the contemporaneous grant amount. If the grant amount"
	local mynotes2 = "differs from the (rounded) MAP amount or if it is zero, we impute size from adjacent months. To enhance readability, the"
	local mynotes3 = "few earnings above `trimearnq' are dropped. All sizes up to 8, all kidcount, JF-Treatment Units, Q1-Q7."
	//--------------------- All together with notes
	graph twoway histogram eCbin if  ///
	            (afdcon_nminq==0|afdcon_nminq==3)&earnq~=0&eCbin>=-`nbinsaroundnotch'*100&eCbin<=`nbinsaroundnotch'*100, ///
				by(afdcon_nminq, ///
					total col(3)  ///
					title("Local Distribution of Quarterly Earnings" "centered at FPL", size(medium)) ///
					graphregion(color(white)) bgcolor(white) ///
					note("`mynotes'" "`mynotes2'" "`mynotes3'", size(vsmall)) ///
				)  ///
				xtitle("{c $|}100-wide bins with bin 0 containing the quartely FPL for JF's eligibility test.", size(small)) ///
				frequency discrete xline(0) ///
				name(`whatwedo', replace) fcolor(none) lcolor(black)
	graph export "${gJF}/`whatwedo'.pdf", logo(off) orientation(landscape) mag(100) replace 
	
	if 1==1 {// open if you want to obtain the individual figures
		// ------------------ Unconditional, no notes
		graph twoway histogram eCbin if ///
					(afdcon_nminq==0|afdcon_nminq==3)&earnq~=0&eCbin>=-`nbinsaroundnotch'*100&eCbin<=`nbinsaroundnotch'*100, ///
						graphregion(color(white)) bgcolor(white) ///
					frequency discrete xline(0) ///
					xtitle("") ///
					name(`whatwedo'_a, replace) fcolor(none) lcolor(black)
		graph export "${gJF}/`whatwedo'_a.pdf", logo(off) orientation(landscape) mag(100) replace 
		// ------------------ On Assistance All 3 Months of the Quarter, no notes
		graph twoway histogram eCbin if  ///
					(afdcon_nminq==3)&earnq~=0&eCbin>=-`nbinsaroundnotch'*100&eCbin<=`nbinsaroundnotch'*100, ///
						graphregion(color(white)) bgcolor(white) ///
					frequency discrete xline(0) ///
					xtitle("") ///
					name(`whatwedo'_b, replace) fcolor(none) lcolor(black)
		graph export "${gJF}/`whatwedo'_b.pdf", logo(off) orientation(landscape) mag(100) replace
		// ------------------ Off Assistance All 3 Months of the Quarter, no notes
		graph twoway histogram eCbin if  ///
					(afdcon_nminq==0)&earnq~=0&eCbin>=-`nbinsaroundnotch'*100&eCbin<=`nbinsaroundnotch'*100, ///
						graphregion(color(white)) bgcolor(white) ///
					frequency discrete xline(0) ///
					xtitle("") ///
					name(`whatwedo'_c, replace)  fcolor(none) lcolor(black)
		graph export "${gJF}/`whatwedo'_c.pdf", logo(off) orientation(landscape) mag(100) replace
	}
end



	
	

	//------------------------------------------------ PART 5  ------------------------------------------------------
	// Execution
	//---------------------------------------------------------------------------------------------------------------
	DistrQEarnings "`jf_dtain'" "`rules_dtain'"
	erase "$ddJF/JF_ADFCrules_temp.dta"

	
	
	
	
	