//--------------------------------------------------------------------------------------------- DistributionOverStates
// This code produces the content of Table 4 (Article) and Tables A6 and A8 (Online Appendix),
// that is, it computes the distribution over observable states (earning / welfare participation).
// 
// This code is embedded into a program to make the output (written to STATA Results' window) more readable.
//
// This code has five parts:
// 	PART 1: sets parameters/ preferences which determine which Table to reproduce, which part of the code to run etc.
// 	PART 2: works on the data in entry to prepare it for PART 3
// 	PART 3: computes the distribution over observable states
// 	PART 4: computes the SEs by bootstrapping 
// 	PART 5: outputs to .txt file the distribution over observable states & the boostrapped data for processing in Matlab
// 	PART 5: execution
// 
// This code take as inputs:
// 	1. pre-generated MDRC data
// 	2. pre-generated data sets of welfare policy rules 
// 
// This code outputs:
// 	1. A .txt file with the distribution over observable states & the boostrapped data for processing in Matlab
// 	2. A .log file with the output appearing on STATA's Results window.
// 
// Last Revision: 09/18/2015
//-------------------------------------------------------------------------------------------------------------------------------
args jf_dtain rules_dtain 





capture program drop DistributionOverStates
capture program define DistributionOverStates
args jf_dtain rules_dtain

	//------------------------------------------------ PART 1  ------------------------------------------------------
	// Choose what you want to do by setting a few local variables.
	// Call .do file containing two programs used below.
	//---------------------------------------------------------------------------------------------------------------

		
		//-----------------------------------------------------------------------
		// Pick the tasks that you want to run by toggling on and off three indicators:
		// ind_distri = 1 if want to get distribution over obs states in the data, otherwise 0
		// ind_bootst = 1 if want to get SEs for distribution over obs states in the data, otherwise 0
		// ind_matlab = 1 if want to write to .txt file used by Matlab code, otherwise 0 
		//-----------------------------------------------------------------------
		local ind_distri = 1
		local ind_bootst = 1 // warning: this is lengthy!
		local ind_matlab = 1
		//-----------------------------------------------------------------------
		// Pick the option concerning how to determine AU size (hence the 
		// FPL value to use to define the earning ranges for each case).
		// The options are {nextsizeup, twosizesup, exactsize}:
		//	- exactsize : uses FPL for AU Size 
		//	- nextsizeup: uses FPL for AU Size + 1
		//	- twosizesup: uses FPL for AU Size + 2
		//-----------------------------------------------------------------------
		local whichFPL "nextsizeup" 
		//-----------------------------------------------------------------------
		// Pick the earning ranges. The options are {FPL,FPL_split2per"}:
		//	FPL				: earnings equal to 0 (range 0), up to FPL (range 1), above (range 2)
		//	FPL_split2perc	: earnings equal to 0 (range 0), up to FPL (range 1), FPL to FPL*(1+%x) (range 2), above (range 3)
		// Set the x% used in option whichthreshold=FPL_split2perc.
		//------------------------------------------------------------------------
		local whichthreshold = "FPL"  
		local split2_percent = "0.2"  
		//-----------------------------------------------------------------------
		// Pick the number of bootstraps for the computation of the SE.
		//-----------------------------------------------------------------------
		local nboots = 1000
		//-----------------------------------------------------------------------
		// Pick which postRA quarters you want to use (among those in the reshaped data)
		//-----------------------------------------------------------------------
		local fq = 1
		local lq = 7


		
		
		
		//-----------------------------------------------------------------------
		// Use the choices made above to:
		// - Define the list of variables used to describe the distribution over states.
		// - Assign informative names to the files produced below.
		//-----------------------------------------------------------------------
		global marginalvars "p0N p1N p2N p0P p1P p2P"
		if "`whichthreshold'"=="FPL_split2perc" {
			global marginalvars "p0N p1N p2N p3N p0P p1P p2P p3P"
		}	
		//----------------------------------------------------------------------- 
		if "`whichFPL'"=="nextsizeup" & "`whichthreshold'" == "FPL" {
			local whatwedo 	"Table4"	
		}
		else if "`whichFPL'"=="nextsizeup" & "`whichthreshold'" == "FPL_split2perc" {
			local whatwedo 	"TableA8"	
		}
		else if "`whichFPL'"=="exactsize"  & "`whichthreshold'" == "FPL" {
			local whatwedo 	"TableA6a"	
		}
		else if "`whichFPL'"=="twosizesup" & "`whichthreshold'" == "FPL" {
			local whatwedo 	"TableA6b"	
		}
		else {
			di "Your choice of options is not reproducing any of the Tables in the article or the Appendix."
			STOP
		}
		local logfileout	"$oJF/`whatwedo'.log"		// for keeping track
		local dtafileout 	"$ddJF/`whatwedo'.dta" 		// for storage
		local dtafileout_bs "$ddJF/`whatwedo'_bs.dta" 	// for boostrapped SEs	
		local txtfileout 	"$ddJF/`whatwedo'_mat.txt" 	// for Matlab processing
		//-----------------------------------------------------------------------
		// Load programs DistributionOverStates and DistributionOverStates_DS,
		// they are called in PARTS 3 and 4 below.
		//-----------------------------------------------------------------------
		do DistributionOverStates_programs

		
		
		
		
		
		
		
		
		
		
		
		
	//------------------------------------------------ PART 2  ------------------------------------------------------
	// Prepare the data for the computations done in PART 3.
	//---------------------------------------------------------------------------------------------------------------
		matrix drop _all
		capture log close
		log using "`logfileout'", replace
		
		
		//-------------------------------------------------------------------
		// Extracting Policy Parameters used below from data base of policy information.
		//-------------------------------------------------------------------
		qui use "`rules_dtain'", clear
		keep b1_nhs b0_nhs F3 year size
		rename F3 F3_
		rename b1_nhs b1_nhs_
		rename b0_nhs b0_nhs_
		qui reshape wide b0_nhs b1_nhs F3, i(year) j(size)
		sort year 
		qui save "$ddJF/JF_ADFCrules_temp.dta", replace
		
		//-------------------------------------------------------------------
		// Getting the data set ready for computing distribution over observed states.
		//-------------------------------------------------------------------
		if `ind_distri'==1 | `ind_bootst'==1 {
			//-------------------------------------------------
			// Create the variables that we want to describe
			//-------------------------------------------------
			qui use ernq0-ernq7 afdconq1-afdconq7 id e ///
					earnq afdcon_nminq afdcon afdcamt year quarter month applcant ///
					$pscorevars $mdrcvars  ///
					kidcount using "`jf_dtain'" if quarter>=`fq' & quarter<=`lq' & kidcount~=., clear

			qui gen nm = .
			sort id quarter month
			qui by id quarter: replace nm = _n

			
			//---------------------------------------------------------------------------------------------- 
			// AU size which is needed in order to associate the correct FPL.
			// We retain cases with kidcount not missing.
			// We add a mother hence we let the AU size (variable size) equal kidcount+1;
			// if kidcount=0 we treat is as if it was =1.
			//--------------------------------------------------------------------------------------	
			qui gen size = kidcount +1 if kidcount==1|kidcount==2|kidcount==3
			qui replace size = 2 if kidcount==0
			//---------------------------------------------------
			// Merge in policy parameters.
			// Drop years for which you do not have the rules (2 obs, 1999 and 2000)
			//---------------------------------------------------
			sort year
			qui merge m:1 year using  "$ddJF/JF_ADFCrules_temp.dta"
			qui drop if _m~=3  
			drop _m
			
			//----------------------------------------------------------------------------------------------
			// For each TREATMENT & CONTROL observation, determine to which bin the quarterly FPL -for recipients- belongs.
			// The variable created is called Cbin because originally it referred to a point in a picture of the budget set under JF.
			//
			// If whichthreshold=="FPL_split2perc" then the variable Dbin is also filled in order to split
			// the range of earnings above the threshold (FPL in this case) into 2 sub-ranges.
			//----------------------------------------------------------------------------------------------
			local maxs = 10		// largest AU size handled
			local maxy = 1999 	// large enough calendar year to go to Q7 post RA
			qui gen Cbin = .
			qui gen Dbin = . 	// default, larger than any number
			qui gen agrantatFPL = .
			//-----------------------
			forvalues y=1996(1)`maxy' {
				forvalues s=1(1)`maxs' {
					//-----------------------------------------
					// This is the size to use for the FPL, = `s' 
					// means exactly the size produced by kidcount+1, 
					// `s'+1 means the next size up etc.
					//-----------------------------------------
					local ss = `s' 
					if "`whichFPL'"=="nextsizeup" {
						local ss = `s' +1
					}
					else if "`whichFPL'"== "twosizesup" {
						local ss = `s' +2
					}
					//-----------------------------------------
					qui gen F_`ss' = F3_`ss'
					//-----------------------------------------
					// Identify the number of the bin to which the 
					// quartely FPL for an AU of this size in this calendar year
					//-----------------------------------------
					qui replace Cbin =  0 if size==`s'&year==`y'&3*F_`ss'==0 				// for completeness, will never be active
					qui replace Cbin =  1 if size==`s'&year==`y'&3*F_`ss'>0&3*F_`ss'<150 	// for completeness
					qui replace Cbin =  round(3*F_`ss',100) if size==`s'&year==`y'&Cbin==.
					//-----------------------------------------
					// Grant amount that the person would receive if she earned exactly FPL
					// under AFDC and if her history was such that she is still subject to both
					// the unreduced fixed and the proportional disregards. The formula is obtained as follows:
					// GRANT = 0.73*(MBS-(earnings - earning reduction)) (with earnings set to FPL)
					// with 
					// earning reduction = (1/3)*(earnings-120)+120   (with earnings set to FPL)
					// so with some algebra:
					// earnings - earning reduction = FPL - (1/3)*(FPL-120)+120 = (2/3)*(FPL-120)
					//-----------------------------------------
					local atao = 0.73
					qui replace agrantatFPL = `atao'*(b1_nhs_`ss' - (2/3)*(F_`ss'-120)) if size==`s'&year==`y' 	// monthly
					qui replace agrantatFPL = 50 if agrantatFPL>0&agrantatFPL<50& size==`s'&year==`y' 			// rounded to $50
					qui replace agrantatFPL = round(agrantatFPL,50) 					if size==`s'&year==`y' 	// rounded to $50
					//----------------------------------------		
					if "`whichthreshold'"=="FPL_split2perc"  { 
						//------------------------------------------
						// Here we use FPL to define range 1, then 
						// split range 2 (above FPL) according to rule (B).
						//
						// Identify the number of the bin to which the quartely FPL * (1+ %)
						// for an AU of this size in this calendar year
						//-----------------------------------------
						qui replace Dbin =  0 if size==`s'&year==`y'&3*F_`ss'*(1+`split2_percent')==0 // for completeness, will never be active
						qui replace Dbin =  1 if size==`s'&year==`y'&3*F_`ss'*(1+`split2_percent')>0&3*F_`ss'*(1+`split2_percent')<150 // for completeness
						qui replace Dbin =  round(3*F_`ss'*(1+`split2_percent'),100) if size==`s'&year==`y'&Dbin==.
					}
					//----------------------------------------
					drop F_`ss'
				}
			}
			assert Cbin~=. if size<=`maxs'&year<=`maxy'		
			//---------------------------------------------------
			// Assign every observation to an earning bin {0,1,2} or {0,1,2,3}:
			//
			// Basic break-down:
			// where 0= q. earning are exactly zero
			// where 1= q. earning are strictly positive and equal to FPL or below where FPL 
			//             is the one used for an AU of this size under JF-TANF in Q1-Q7
			// where 2= q. earning are strictly above FPL 
			//
			// Split-breakdonw (break down only range 2):
			// where 0= q. earning are exactly zero
			// where 1= q. earning are strictly positive and equal to FPL or below where FPL 
			//             is the one used for an AU of this size under JF-TANF in Q1-Q7
			// where 2= q. earning are strictly above FPL but not too much
			// where 3= q. earning are markedly above FPL 
			//---------------------------------------------------		
			qui gen ebin 	 = 0 if earnq==0
			qui replace ebin = 1 if earnq<=Cbin	& earnq~=0
			qui replace ebin = 2 if earnq>Cbin	& earnq~=.
			if "`whichthreshold'"=="FPL_split2perc" {
				qui replace ebin = 3 if ebin ==2 & earnq>Dbin  
			}		
			//---------------------------------------------------
			// Here we:
			// - Toggle all quarters that are used in computing distribution over states:
			//   we include only quarters such that the case is always on (or always off) welfare assistance.
			// - Assigne every observation to a (welfare) participation bin.
			// - Given the income bin and welfare participation, fill the 0/1 dummies for each state.
			//---------------------------------------------------
			qui gen included = (afdcon_nminq==0|afdcon_nminq==3) 
			qui gen partic = afdcon_nminq
			qui replace partic = 1 if afdcon_nminq==3
			qui replace partic = . if included==0	
			
			//------------------------------------
			qui gen p0N = (ebin==0&partic==0)
			qui gen p1N = (ebin==1&partic==0)
			qui gen p2N = (ebin==2&partic==0)
			qui gen p0P = (ebin==0&partic==1)
			qui gen p1P = (ebin==1&partic==1)
			qui gen p2P = (ebin==2&partic==1)
			qui gen p3N = (ebin==3&partic==0) // active only when whichthreshold=="FPL_split2perc"
			qui gen p3P = (ebin==3&partic==1) // active only when whichthreshold=="FPL_split2perc"
			//------------------------------------
			sort id year quarter month
			qui by id year quarter: egen afdcamtq = sum(afdcamt)
			qui gen p2R = (p2P==1&afdcamtq<=3*agrantatFPL)
			qui replace p2R = 0 if included==0
			drop afdcamtq
			drop agrantatFPL
			//------------------------------------
			foreach var of varlist $marginalvars {
				qui replace `var' = 0 if included==0
			}
			//---------------------------------------------------
			// Wlog we drop 2 of the 3 months in the quarter. 
			//---------------------------------------------------
			qui gen nn = .
			qui bys id quarter: replace nn = _n
			qui drop if nn~=1 
			//-------------------------------------------------- 
			// Get a few figures reported in Section 6, subsection "Unreduced Disregards".
			// - Count the quarters in which the classification is p2R (control sample)
			// - Count the quarters in total (control sample)
			//--------------------------------------------------
			di ""
			di ""
			di " ====================================== Empirical Relevance of State 2R"
			di " (result referenced in Section VI. Subsection A. of the paper)"
			qui count if p2R==1&e==0&included==1
			di "The number of (control) quarters in which the state is 2R is =" r(N)
			qui count if e==0&included==1
			di "out of these many (control) quarters =" r(N)
					
			//----------------------------------------------------- 
			keep ernq0-ernq7 afdconq1-afdconq7 id e ///
					quarter applcant  included $pscorevars $mdrcvars $marginalvars 			
			qui reshape wide $marginalvars included, i(id) j(quarter)
			//---------------------------------------------------
			// Saving temporary data set.
			//---------------------------------------------------
			qui save "$ddJF/temp_jf_dtain.dta", replace
		}

		

		
		
		


	//------------------------------------------------ PART 3  ------------------------------------------------------
	// Here you use the data to compute the distribution over states.
	// Distribution is written to STATA Results' window, as well as recorded into the (open) log file.
	// The processing is embedded into an internal program (DistributionOverStates) for convenience.
	//---------------------------------------------------------------------------------------------------------------
		//-------------------------------------------------
		if `ind_distri'==1  {
			DistributionOverStates_P "$ddJF/temp_jf_dtain.dta"  "`dtafileout'" 
		}
		
		
			
			
			
			

	//------------------------------------------------ PART 4  ------------------------------------------------------
	// Here you bootstrap the data to obtain SEs for the statistics obtained in PART 3.
	// SEs are written to STATA Results' window, as well as recorded into the (open) log file.
	// The processing is embedded into an internal program (DistributionOverStates_SE) for convenience.
	//---------------------------------------------------------------------------------------------------------------
		if  `ind_bootst'==1 {
			set seed 112233
			qui use "$ddJF/temp_jf_dtain.dta", clear
			DistributionOverStates_SE `nboots' "`dtafileout_bs'"  
		}
		//-------------------------------------------------
		capture log close

		
		
		
		
		

	//------------------------------------------------ PART 5  ------------------------------------------------------
	// Here you write to .txt file the distribution over states in the data (1st row) and the 
	// distributions over states in the boostrapped data (2nd row till the end) for further processing in Matlab.
	//---------------------------------------------------------------------------------------------------------------
		if `ind_matlab'==1 {
			//----------------------------------------- 
			// Some preliminaries
			//-----------------------------------------
			if "$marginalvars"=="p0N p1N p2N p0P p1P p2P" { 
				local matlabv "p0N_t1	p0N_c1	p1N_t1	p1N_c1	p2N_t1	p2N_c1"
				local matlabv "`matlabv' p0P_t1	p0P_c1	p1P_t1	p1P_c1	p2P_t1	p2P_c1"
			}
			else if  "$marginalvars"=="p0N p1N p2N p3N p0P p1P p2P p3P" {
				local matlabv "p0N_t1	p0N_c1	p1N_t1	p1N_c1	p2N_t1	p2N_c1 p3N_t1	p3N_c1"
				local matlabv "`matlabv' p0P_t1	p0P_c1	p1P_t1	p1P_c1	p2P_t1	p2P_c1 p3P_t1	p3P_c1"
			}
			else {
				di "Not coded"
				STOP
			}
			//-----------------------------------------
			// Write to txt file in the order required by Matlab code
			//-----------------------------------------
			qui use "`dtafileout'", clear
			qui append using "`dtafileout_bs'"
			qui outsheet `matlabv' using "`txtfileout'", replace
		}	
	
	//---------------------------------------------
	erase "$ddJF/temp_jf_dtain.dta"
	erase "$ddJF/JF_ADFCrules_temp.dta"
	erase "`dtafileout'"
	erase "`dtafileout_bs'"
end




//------------------------------------------------ PART 6  ------------------------------------------------------
// Execute
//---------------------------------------------------------------------------------------------------------------
DistributionOverStates "`jf_dtain'" "`rules_dtain'"




