//------------------------------------------------------------------------- MeanSampleCharacteristics
// This code produces the content of Table 2 (Article), that is, it computes Mean Sample Characteristics.
// 
// This code is embedded into a program to make the output (written to STATA Results' window) more readable.
//
// This code has five parts:
// 	PART 1: sets parameters/ preferences which determine which part of the code to run etc.
// 	PART 2: works on the data in entry to prepare it for PART 3
// 	PART 3: computes mean characteristics and differences in mean characteristics for overall sample and subsamples
// 	PART 4: computes the SEs of the differences in mean characteristics obtained in PART 3 by bootstrapping 
//	PART 5: send code for execution 
//
// This code take as inputs:
// 	1. pre-generated MDRC data
// 
// This code outputs:
// 	1. Three .dta files with the boostrapped differences in mean characteristics, if desired
// 	2. One .log file with the output appearing on STATA's Results window.
// 
// Last Revision: 09/04/2015
//-------------------------------------------------------------------------------------------------------------------------------
args jf_dtain  


	

	
	
	
	
capture program drop MeanSampleCharacteristics
capture program define MeanSampleCharacteristics
args jf_dtain

	//------------------------------------------------ PART 1  ------------------------------------------------------
	// Choose what you want to do by setting a few local variables.
	//---------------------------------------------------------------------------------------------------------------

		
		//-----------------------------------------------------------------------
		// Pick the tasks that you want to run by toggling on and off three indicators:
		// 		ind_distri = 1 if want to get mean sample characteristics in the data, otherwise 0
		// 		ind_bootst = 1 if want to get SEs for the mean sample characteristics, otherwise 0
		// 		ind_bootsv = 1 if want to save the boostrapped differences in mean characteristics, otherwise 0
		//
		// Pick the number of bootstraps for the computation of the SE.
		//-----------------------------------------------------------------------
		local ind_descst = 1
		local ind_bootst = 1 // warning: this task can be lenghty
		local ind_bootsv = 0
		local nboots = 1000

		//-----------------------------------------------------------------------
		// Assign informative names to the files produced below.
		//-----------------------------------------------------------------------
		local whatwedo "Table2"
		local logfileout		"$oJF/`whatwedo'.log"		// for keeping track
		local dtafileout_all	"$ddJF/`whatwedo'_all.dta" 	// for storage
		local dtafileout_g0 	"$ddJF/`whatwedo'_g0.dta" 	// for storage
		local dtafileout_g1 	"$ddJF/`whatwedo'_g1.dta" 	// for storage

		

		
		
	matrix drop _all
	capture log close
	log using "`logfileout'", replace	
	//------------------------------------------------ PART 2  ------------------------------------------------------
	// Obtain the descriptive statistics in the Table for Overall Sample and 
	// two pre RA earning groups, those with zero earnings during the 7 quarters pre RA (earning group 0) 
	// and those with some earnings during the same period (earning group 1).
	//---------------------------------------------------------------------------------------------------------------	
		if `ind_descst'==1 {
			//-----------------------------------------------------------------
			// Load the data:
			// -	By picking month=0 we effectively use wide-format data.
			// -	Implement sample exclusions based on kidcount.
			//-----------------------------------------------------------------
			qui use "`jf_dtain'" if month==0&kidcount~=., clear 
			//-------------------------------------------------------------------
			// Create vars that identify the subgroups we care about (see LocalPrograms.do)
			//-------------------------------------------------------------------
			create_earning_subgroup 
			//--------------------------------------------------------------------
			// Run pscore logit, predict propensity score (pscore), and generate IPW (pscorewt)
			// for each pre RA earning subgroup (based on earnings in 7th Q prior to RA)
			//--------------------------------------------------------------------
			drop pscorewt
			qui logit e ${pscorevars} , asis
			qui predict double pscore                                      
			gen double pscorewt = e/pscore + (1-e)/(1-pscore) 
			drop pscore	
			foreach g of numlist  0 1 {
				qui count if earning_subgroup==`g'
				qui logit e ${pscorevars_g`g'} if earning_subgroup==`g', asis
				qui predict double pscore                                      
				qui gen double pscorewt_e`g' = e/pscore + (1-e)/(1-pscore) if earning_subgroup==`g'
				drop pscore
			}
			//--------------------------------------------------------------------
			// Temporarily undo the missing replacement done at the stage 
			// of data creation in order to draw a table like that in BGH (2005, AER):
			// misshs ==> nohsged  hsged 
			// misskidctgt2 ==> kidctgt2
			// missmar ==> marnvr marapt
			//--------------------------------------------------------------------
			qui for var nohsged hsged mthsgrad : replace X=. if misshs==1
			qui replace kidctgt2=. if misskidctgt2==1
			qui for var marnvr marapt : replace X=. if missmar==1
			
			
			
			
			//-------------------------------------------------------------------
			// Obtain the figures in the Table for Overall Sample
			//-------------------------------------------------------------------
				//-------------------------------------------------------------------
				// Compute Standard Deviations appearing in square brakets in the Table
				//-------------------------------------------------------------------
				matrix define sd = J($nv,2,.)
				local nv = 0
				foreach var of varlist $mdrcvars {
					local nv = `nv'+1
					qui sum `var' if e==1
					matrix define sd[`nv',1] = r(sd)
					qui sum `var' if e==0
					matrix define sd[`nv',2] = r(sd) 		
				}
				matrix define sd = sd'
				//-------------------------------------------------------------------
				// Compute unadjusted and adjusted means appearing in the Table.
				// We use BGH's meandiff function (the unadjusted means for Jobs First group: eqname t;
				// those for AFDC group: eqname c' the differences: eqname d).
				//-------------------------------------------------------------------
				qui meandifs $mdrcvars , by(e) // Unadjusted Means
				matrix define Tab= r(table)
				forvalues j=1(1)3 {
					local l = (`j'-1)*$nv+1
					local u = `j'*$nv
					matrix define b`j' = Tab[1,`l'..`u']
				}
				qui meandifs $mdrcvars [aw=pscorewt], by(e)  // Adjusted Means
				matrix define Tab= r(table)
				forvalues j=1(1)3 {
					local l = (`j'-1)*$nv+1
					local u = `j'*$nv
					matrix define a`j' = Tab[1,`l'..`u']
				}	
				//-------------------------------------------------------------------
				// Store the information in a matrix, for convenience
				//-------------------------------------------------------------------	
				matrix define m_all = (b1\b2\b3\a3\sd)'
				matrix colnames m_all = Tmean Cmean UDiff ADiff Tmean_SD Cmean_SD
				
			
			
			
			//-------------------------------------------------------------------
			// Obtain figures in the Table for Earning subgroups 
			//-------------------------------------------------------------------	
				foreach g of numlist  0 1 {
					//-------------------------------------------------------------------
					// Compute Standard Deviations appearing in square brakets in the Table
					//-------------------------------------------------------------------
					matrix define sd = J($nv,2,.)
					local nv = 0
					foreach var of varlist $mdrcvars {
						local nv = `nv'+1
						qui sum `var' if e==1&earning_subgroup==`g'
						matrix define sd[`nv',1] = r(sd)
						qui sum `var' if e==0&earning_subgroup==`g'
						matrix define sd[`nv',2] = r(sd) 		
					}
					matrix define sd = sd'
					//-------------------------------------------------------------------
					// Compute means appearing in the Table.
					// We use BGH's meandiff function (the unadjusted means for 
					// Jobs First group: eqname t; those for AFDC group: eqname c' the 
					// differences: eqname d.
					//-------------------------------------------------------------------
					qui meandifs $mdrcvars if earning_subgroup==`g', by(e) // Unadjusted Means
					matrix define Tab= r(table)
					forvalues j=1(1)3 {
						local l = (`j'-1)*$nv+1
						local u = `j'*$nv
						matrix define b`j' = Tab[1,`l'..`u']
					}
					qui meandifs $mdrcvars [aw=pscorewt_e`g'] if earning_subgroup==`g', by(e)  // Adjusted Means
					matrix define Tab= r(table)
					forvalues j=1(1)3 {
						local l = (`j'-1)*$nv+1
						local u = `j'*$nv
						matrix define a`j' = Tab[1,`l'..`u']
					}	
					//-------------------------------------------------------------------
					// Store the information in a matrix, for convenience
					//-------------------------------------------------------------------	
					matrix define m_g`g' = (b1\b2\b3\a3\sd)'
					matrix colnames m_g`g' = Tmean Cmean UDiff ADiff Tmean_SD Cmean_SD	
				}
					
			// -------------------------------------------------------------------
			// Write out results (also use for Excel processing)
			// -------------------------------------------------------------------
			di ""
			di ""
			di ""
			di "================== `whatwedo': Means, Differences in Means and SDs ================"
			di ""
			di ""
			di ""			
			di "================== `whatwedo' (Overall Sample) ===================================="
			matrix list m_all, noheader format(%6.3f)
			di ""
			di ""
			di "================== `whatwedo' (Earning group 0 - zero pre RA earnings) ============"
			matrix list m_g0, noheader format(%6.3f)
			di ""
			di ""
			di "================== `whatwedo' (Earning group 1 - positive pre RA earnings) ========"
			matrix list m_g1, noheader format(%6.3f)
				

			// -------------------------------------------------------------------
			// Helling's T-squared generalized test of whether a set of mean is equal b/w two groups
			// -------------------------------------------------------------------
			di ""
			di ""
			di "================== Hotelling's T-squared generalized test (Overall Sample) ========"
			hotelling $mdrcvars, by(e) notable
			//------------------------------------------------------
			foreach g of numlist  0 1 {
				di ""
				di ""
				di "================== Hotelling's T-squared generalized test (Earning subgroup `g') =="
				hotelling $mdrcvars  if earning_subgroup==`g', by(e) notable
			}	
			
		} // end of if ind_descst==1











	//------------------------------------------------ PART 4  ------------------------------------------------------
	// Obtain the boostrapped SEs of the mean differences (Treatment - Control) in sample characteristics 
	// (reported in parenthesis in the Table).
	//---------------------------------------------------------------------------------------------------------------	

		if `ind_bootst'==1 {
			set seed 112233
			di ""
			di ""
			di ""
			di "================== Bootstrapping SEs (Overall Sample) ============================="
			//------------------------------------------------------------------------
			// Work on the overall Sample. Sample exclusions based on kidcount.
			//------------------------------------------------------------------------
			qui use ernq0-ernq7 afdconq1-afdconq7 id e ///
					afdcon quarter month applcant $pscorevars $mdrcvars kidcount ///
					using "`jf_dtain'" if month==0&kidcount~=., clear
			
					
			//------------------------------------------------------------------
			// Bootstrap nboots samples and for each compute difference (T-C) in means
			// for the variables in $mdrcvars.
			//------------------------------------------------------------------
			rename id pufid
			forvalues r=1(1)`nboots' {
				//--------------------------------------------------
				forvalues i=0(1)10 {
					if `r'==`i'*100 {
						di "- working on bootstrapped sample `r' "
					}
				}
				if `r'==1 {
					di "- working on bootstrapped sample `r' (boostrapping takes time ...)"
				}
				//--------------------------------------------------
				preserve
					//--------------------------------------------------
					// Draw r-th random sample.
					//--------------------------------------------------
					qui bsample, cluster(pufid) idcluster(id)
					//---------------------------------------------------
					// Get inverse propensity score weighting (IPW). 
					//---------------------------------------------------
					qui logit e ${pscorevars}  
					qui predict double pscore                                      
					qui gen double pscorewt = e/pscore + (1-e)/(1-pscore)
					drop pscore			 
					//--------------------------------------------------
					// Undo the missing replacement done at the stage of data
					// creation in order to draw a table as in BGH (2005, AER):
					// misshs ==> nohsged  hsged 
					// misskidctgt2 ==> kidctgt2
					// missmar ==> marnvr marapt
					//--------------------------------------------------
					qui for var nohsged hsged mthsgrad : replace X=. if misshs==1
					qui replace kidctgt2=. if misskidctgt2==1
					qui for var marnvr marapt : replace X=. if missmar==1			
					//---------------------------------------------------
					// Compute the differences in means and store them into a matrix
					// which is appended to across bootstrap draws.
					//---------------------------------------------------
					local l = 2*$nv+1
					local u = 3*$nv
					if `r'==1 {
						qui meandifs $mdrcvars, by(e) 
						matrix define TabU = r(table)
						matrix define TabU = TabU[1,`l'..`u']
						qui meandifs $mdrcvars [aw=pscorewt], by(e) 
						matrix define TabA = r(table)
						matrix define TabA = TabA[1,`l'..`u']
						local stored_names :  colnames TabA
					}
					else {
						qui meandifs $mdrcvars, by(e) 
						matrix define TabU2 = r(table)
						matrix define TabU = TabU\TabU2[1,`l'..`u'] // appending
						qui meandifs $mdrcvars [aw=pscorewt], by(e) 
						matrix define TabA2 = r(table)
						matrix define TabA = TabA\TabA2[1,`l'..`u'] // appending
					}
				restore
			} // over boostrapped samples
			

			
			//----------------------------------------------------------------------------------------------
			// Use the bootstrapped differences in means to compute SEs.
			//----------------------------------------------------------------------------------------------
			clear
			//---------------------------
			qui svmat TabU, names(eqcol)
			rename d* * 
			matrix define ser = J($nv,2,.)
			local nv = 0
			foreach var of varlist $mdrcvars {
				local nv = `nv'+1
				qui sum `var' 
				matrix define ser[`nv',1] = r(sd)		
			}	
			rename * *_diffun
			//----------------------------
			qui svmat TabA, names(eqcol)
			rename d* * 
			local nv = 0
			foreach var of varlist $mdrcvars {
				local nv = `nv'+1
				qui sum `var' 
				matrix define ser[`nv',2] = r(sd)		
			}	
			rename * *_diff	
			//----------------------------
			matrix colnames ser = UDiff_SE ADiff_SE
			di ""
			di "================== `whatwedo' (Overall Sample) ===================================="
			if `ind_descst'==1 {
				matrix define `whatwedo' = [m_all,ser]
				di " The bootstrapped SEs for the unadjusted & adjusted diff.s in means appear in the last 2 cols:"
				matrix list `whatwedo', noheader format(%6.3f)
			}
			else {
				matrix rownames ser = `stored_names'
				di " The bootstrapped SEs for the unadjusted & adjusted diff.s in means are:"
				matrix list ser	, noheader format(%6.3f)	
			}
			if `ind_bootsv'==1 {
				qui keep if _n<=`nboots'
				keep *_diff*
				qui save "`dtafileout_all'", replace
			}
			clear
			
		

		
			
			//------------------------------------------------------------------------------------------
			// With reference to the preRA earning groups compute SEs for the differences in mean sample characteristics.
			//------------------------------------------------------------------------------------------
			qui use ernq0-ernq7 afdconq1-afdconq7 id e ///
					afdcon quarter month applcant $pscorevars $mdrcvars kidcount ///
					using "`jf_dtain'" if month==0&kidcount~=., clear	
	
			//-------------------------------------------------------------------
			// Create vars that identify the subgroups 
			//-------------------------------------------------------------------
			create_earning_subgroup
			tempfile file2
			qui save `file2', replace
			//-------------------------------------------------------------------
			foreach g of numlist  0 1 {
				//---------------------------------------------------------------
				qui use `file2' if earning_subgroup==`g' 
				rename id pufid
				//---------------------------------------------------------------
				di ""
				di ""
				di ""
				di "================== Bootstrapping SEs (Earning Subsample `g') ======================"
				forvalues r=1(1)`nboots' {
					//--------------------------------------------------
					forvalues i=0(1)10 {
						if `r'==`i'*100 {
							di "- working on bootstrapped sample `r' "
						}
					}
					if `r'==1 {
						di "- working on bootstrapped sample `r' (boostrapping takes time ...)"
					}
					preserve
						//---------------------------------------------------
						// Draw a random sample from existing data
						//---------------------------------------------------
						qui bsample, cluster(pufid) idcluster(id)
						//---------------------------------------------------
						// Now executing code necessary for pscore model and inverse propensity score weighting (IPW) stuff 
						//---------------------------------------------------
						qui logit e $pscorevars 		// use instead ${pscorevars_g`g'} ??????????????
						qui predict double pscore                                      
						qui gen double pscorewt = e/pscore + (1-e)/(1-pscore)
						drop pscore
						//--------------------------------------------------
						// We need to temporarily undo the missing replacement done
						// at the stage of data creation in order to draw a table like BGH did:
						// misshs ==> nohsged  hsged 
						// misskidctgt2 ==> kidctgt2
						// missmar ==> marnvr marapt
						//--------------------------------------------------
						qui for var nohsged hsged mthsgrad : replace X=. if misshs==1
						qui replace kidctgt2=. if misskidctgt2==1
						qui for var marnvr marapt : replace X=. if missmar==1					
						//---------------------------------------------------
						// Compute and store the differences in means.
						//---------------------------------------------------
						local l = 2*$nv+1
						local u = 3*$nv
						if `r'==1 {
							qui meandifs $mdrcvars, by(e) 
							matrix define TabU = r(table)
							matrix define TabU = TabU[1,`l'..`u']
							qui meandifs $mdrcvars [aw=pscorewt], by(e) 
							matrix define TabA = r(table)
							matrix define TabA = TabA[1,`l'..`u']
							local stored_names :  colnames TabA
						}
						else {
							qui meandifs $mdrcvars, by(e) 
							matrix define TabU2 = r(table)
							matrix define TabU = TabU\TabU2[1,`l'..`u'] // appending
							qui meandifs $mdrcvars [aw=pscorewt], by(e) 
							matrix define TabA2 = r(table)
							matrix define TabA = TabA\TabA2[1,`l'..`u'] // appending
						}
					
					
					restore
				} // loop over boostrapped samples
				//----------------------------------------------------------------------------------------------
				// With reference to the earning subgroup on which you are working:
				// Use the differences in means for each bootstrapped sample to compute SEs.
				//----------------------------------------------------------------------------------------------
				clear
				//---------------------------
				qui svmat TabU, names(eqcol)
				rename d* * 
				matrix define ser = J($nv,2,.)
				local nv = 0
				foreach var of varlist $mdrcvars {
					local nv = `nv'+1
					qui sum `var' 
					matrix define ser[`nv',1] = r(sd)		
				}	
				rename * *_diffun
				//----------------------------
				qui svmat TabA, names(eqcol)
				rename d* * 
				local nv = 0
				foreach var of varlist $mdrcvars {
					local nv = `nv'+1
					qui sum `var' 
					matrix define ser[`nv',2] = r(sd)		
				}	
				rename * *_diff	
				//----------------------------
				matrix colnames ser = UDiff_SE ADiff_SE
				di ""
				di "================== `whatwedo' (Earning SubGROUP `g') ============================="
				if `ind_descst'==1 {
					matrix define `whatwedo' = [m_g`g',ser]
					di " The bootstrapped SEs for the unadjusted & adjusted diff.s in means appear in the last 2 cols:"
					matrix list `whatwedo', noheader format(%6.3f)
				}
				else {
					matrix rownames ser = `stored_names'
					di " The bootstrapped SEs for the unadjusted & adjusted diff.s in means are:"
					matrix list ser	, noheader format(%6.3f)	
				}
				//----------------------------
				if `ind_bootsv'==1 {
					qui keep if _n<=`nboots'
					keep *_diff*
					qui save "`dtafileout_g`g''", replace
				}
				clear			
			} // loop over pre RA earning subgroups
		} // if want to boostrap SEs
		//-------------------------------------------
		log close
		// in released code we erase everything
		capture erase	"`dtafileout_all'" 	
		capture erase 	"`dtafileout_g0'" 	
		capture erase 	"`dtafileout_g1'" 			
		
end





	//------------------------------------------------ PART 5  ------------------------------------------------------
	// Execution
	//---------------------------------------------------------------------------------------------------------------
	MeanSampleCharacteristics "`jf_dtain'"

	
	
	

